@@ -6,51 +6,52 @@
 #' @param num_class Number of classes.
 #' @template roxlate-ml-probabilistic-classifier-params
 #' @export
-xgboost_classifier <- function(x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
-                               min_child_weight = 1, max_delta_step = 0,
-                               grow_policy = "depthwise", max_bins = 16, subsample = 1,
-                               colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
-                               alpha = 0, tree_method = "auto", sketch_eps = 0.03,
-                               scale_pos_weight = 1, sample_type = "uniform",
-                               normalize_type = "tree", rate_drop = 0, skip_drop = 0,
-                               lambda_bias = 0, tree_limit = 0, num_round = 1,
-                               num_workers = 1, nthread = 1, use_external_memory = FALSE,
-                               silent = 0, custom_obj = NULL, custom_eval = NULL,
-                               missing = NaN, seed = 0, timeout_request_workers = 30 * 60 * 1000,
-                               checkpoint_path = "", checkpoint_interval = -1,
-                               objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
-                               num_early_stopping_rounds = 0, objective_type = "classification",
-                               eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
-                               base_margin_col = NULL,
-                               thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
-                               prediction_col = "prediction", probability_col = "probability",
-                               raw_prediction_col = "rawPrediction",
-                               uid = random_string("xgboost_classifier_"), ...) {
+xgboost_classifier <- function(
+    x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
+    min_child_weight = 1, max_delta_step = 0,
+    grow_policy = "depthwise", max_bins = 16, subsample = 1,
+    colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
+    alpha = 0, tree_method = "auto", sketch_eps = NULL,
+    scale_pos_weight = 1, sample_type = "uniform",
+    normalize_type = "tree", rate_drop = 0, skip_drop = 0,
+    lambda_bias = 0, tree_limit = 0, num_round = 1,
+    num_workers = 1, nthread = 1, use_external_memory = FALSE,
+    silent = 0, custom_obj = NULL, custom_eval = NULL,
+    missing = NaN, seed = 0, timeout_request_workers = NULL,
+    checkpoint_path = "", checkpoint_interval = -1,
+    objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
+    num_early_stopping_rounds = 0, objective_type = "classification",
+    eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
+    base_margin_col = NULL,
+    thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
+    prediction_col = "prediction", probability_col = "probability",
+    raw_prediction_col = "rawPrediction",
+    uid = random_string("xgboost_classifier_"), ...) {
   UseMethod("xgboost_classifier")
 }
 
 #' @export
-xgboost_classifier.spark_connection <- function(x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
-                                                min_child_weight = 1, max_delta_step = 0,
-                                                grow_policy = "depthwise", max_bins = 16, subsample = 1,
-                                                colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
-                                                alpha = 0, tree_method = "auto", sketch_eps = 0.03,
-                                                scale_pos_weight = 1, sample_type = "uniform",
-                                                normalize_type = "tree", rate_drop = 0, skip_drop = 0,
-                                                lambda_bias = 0, tree_limit = 0, num_round = 1,
-                                                num_workers = 1, nthread = 1, use_external_memory = FALSE,
-                                                silent = 0, custom_obj = NULL, custom_eval = NULL,
-                                                missing = NaN, seed = 0, timeout_request_workers = 30 * 60 * 1000,
-                                                checkpoint_path = "", checkpoint_interval = -1,
-                                                objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
-                                                num_early_stopping_rounds = 0, objective_type = "classification",
-                                                eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
-                                                base_margin_col = NULL,
-                                                thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
-                                                prediction_col = "prediction", probability_col = "probability",
-                                                raw_prediction_col = "rawPrediction",
-                                                uid = random_string("xgboost_classifier_"), ...) {
-
+xgboost_classifier.spark_connection <- function(
+    x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
+    min_child_weight = 1, max_delta_step = 0,
+    grow_policy = "depthwise", max_bins = 16, subsample = 1,
+    colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
+    alpha = 0, tree_method = "auto", sketch_eps = NULL,
+    scale_pos_weight = 1, sample_type = "uniform",
+    normalize_type = "tree", rate_drop = 0, skip_drop = 0,
+    lambda_bias = 0, tree_limit = 0, num_round = 1,
+    num_workers = 1, nthread = 1, use_external_memory = FALSE,
+    silent = 0, custom_obj = NULL, custom_eval = NULL,
+    missing = NaN, seed = 0, timeout_request_workers = NULL,
+    checkpoint_path = "", checkpoint_interval = -1,
+    objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
+    num_early_stopping_rounds = 0, objective_type = "classification",
+    eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
+    base_margin_col = NULL,
+    thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
+    prediction_col = "prediction", probability_col = "probability",
+    raw_prediction_col = "rawPrediction",
+    uid = random_string("xgboost_classifier_"), ...) {
   args <- list(
     eta = eta,
     gamma = gamma,
@@ -106,6 +107,8 @@ xgboost_classifier.spark_connection <- function(x, formula = NULL, eta = 0.3, ga
 
   args <- validator_xgboost_classifier(args)
 
+  xg_unsupported(args)
+
   stage_class <- "ml.dmlc.xgboost4j.scala.spark.XGBoostClassifier"
 
   jobj <- sparklyr::spark_pipeline_stage(
@@ -119,16 +122,16 @@ xgboost_classifier.spark_connection <- function(x, formula = NULL, eta = 0.3, ga
     raw_prediction_col = args[["raw_prediction_col"]]
   ) %>%
     invoke("setAlpha", args[["alpha"]]) %>%
-    sparklyr::jobj_set_param("setBaseMarginCol", args[["base_margin_col"]]) %>%
+    jobj_set_param("setBaseMarginCol", args[["base_margin_col"]]) %>%
     invoke("setBaseScore", args[["base_score"]]) %>%
     invoke("setCheckpointInterval", args[["checkpoint_interval"]]) %>%
     invoke("setCheckpointPath", args[["checkpoint_path"]]) %>%
     invoke("setColsampleBylevel", args[["colsample_bylevel"]]) %>%
     invoke("setColsampleBytree", args[["colsample_bytree"]]) %>%
-    sparklyr::jobj_set_param("setCustomEval", args[["custom_eval"]]) %>%
-    sparklyr::jobj_set_param("setCustomObj", args[["custom_obj"]]) %>%
+    jobj_set_param("setCustomEval", args[["custom_eval"]]) %>%
+    jobj_set_param("setCustomObj", args[["custom_obj"]]) %>%
     invoke("setEta", args[["eta"]]) %>%
-    sparklyr::jobj_set_param("setEvalMetric", args[["eval_metric"]]) %>%
+    jobj_set_param("setEvalMetric", args[["eval_metric"]]) %>%
     invoke("setGamma", args[["gamma"]]) %>%
     invoke("setGrowPolicy", args[["grow_policy"]]) %>%
     invoke("setLambda", args[["lambda"]]) %>%
@@ -140,55 +143,56 @@ xgboost_classifier.spark_connection <- function(x, formula = NULL, eta = 0.3, ga
     invoke("setMinChildWeight", args[["min_child_weight"]]) %>%
     invoke("setNormalizeType", args[["normalize_type"]]) %>%
     invoke("setNthread", args[["nthread"]]) %>%
-    sparklyr::jobj_set_param("setNumClass", args[["num_class"]]) %>%
+    jobj_set_param("setNumClass", args[["num_class"]]) %>%
     invoke("setNumEarlyStoppingRounds", args[["num_early_stopping_rounds"]]) %>%
     invoke("setNumRound", args[["num_round"]]) %>%
     invoke("setNumWorkers", args[["num_workers"]]) %>%
-    sparklyr::jobj_set_param("setObjective", args[["objective"]]) %>%
+    jobj_set_param("setObjective", args[["objective"]]) %>%
     invoke("setObjectiveType", args[["objective_type"]]) %>%
     invoke("setRateDrop", args[["rate_drop"]]) %>%
     invoke("setSampleType", args[["sample_type"]]) %>%
     invoke("setScalePosWeight", args[["scale_pos_weight"]]) %>%
     invoke("setSeed", args[["seed"]]) %>%
     invoke("setSilent", args[["silent"]]) %>%
-    invoke("setSketchEps", args[["sketch_eps"]]) %>%
     invoke("setSkipDrop", args[["skip_drop"]]) %>%
     invoke("setSubsample", args[["subsample"]]) %>%
-    sparklyr::jobj_set_param("setThresholds", args[["thresholds"]]) %>%
-    invoke("setTimeoutRequestWorkers", args[["timeout_request_workers"]]) %>%
+    jobj_set_param("setThresholds", args[["thresholds"]]) %>%
     invoke("setTrainTestRatio", args[["train_test_ratio"]]) %>%
     invoke("setTreeMethod", args[["tree_method"]]) %>%
     invoke("setUseExternalMemory", args[["use_external_memory"]]) %>%
-    sparklyr::jobj_set_param("setWeightCol", args[["weight_col"]])
+    jobj_set_param("setWeightCol", args[["weight_col"]])
 
   if (!is.nan(args[["missing"]])) {
-    jobj <- sparklyr::invoke_static(x, "sparkxgb.Utils", "setMissingParam", jobj, args[["missing"]])
+    jobj <- sparklyr::invoke_static(
+      x, "sparkxgb.Utils", "setMissingParamClass", jobj, args[["missing"]]
+    )
   }
 
   new_xgboost_classifier(jobj)
 }
 
 #' @export
-xgboost_classifier.ml_pipeline <- function(x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
-                                           min_child_weight = 1, max_delta_step = 0,
-                                           grow_policy = "depthwise", max_bins = 16, subsample = 1,
-                                           colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
-                                           alpha = 0, tree_method = "auto", sketch_eps = 0.03,
-                                           scale_pos_weight = 1, sample_type = "uniform",
-                                           normalize_type = "tree", rate_drop = 0, skip_drop = 0,
-                                           lambda_bias = 0, tree_limit = 0, num_round = 1,
-                                           num_workers = 1, nthread = 1, use_external_memory = FALSE,
-                                           silent = 0, custom_obj = NULL, custom_eval = NULL,
-                                           missing = NaN, seed = 0, timeout_request_workers = 30 * 60 * 1000,
-                                           checkpoint_path = "", checkpoint_interval = -1,
-                                           objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
-                                           num_early_stopping_rounds = 0, objective_type = "classification",
-                                           eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
-                                           base_margin_col = NULL,
-                                           thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
-                                           prediction_col = "prediction", probability_col = "probability",
-                                           raw_prediction_col = "rawPrediction",
-                                           uid = random_string("xgboost_classifier_"), ...) {
+xgboost_classifier.ml_pipeline <- function(
+    x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
+    min_child_weight = 1, max_delta_step = 0,
+    grow_policy = "depthwise", max_bins = 16, subsample = 1,
+    colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
+    alpha = 0, tree_method = "auto", sketch_eps = NULL,
+    scale_pos_weight = 1, sample_type = "uniform",
+    normalize_type = "tree", rate_drop = 0, skip_drop = 0,
+    lambda_bias = 0, tree_limit = 0, num_round = 1,
+    num_workers = 1, nthread = 1, use_external_memory = FALSE,
+    silent = 0, custom_obj = NULL, custom_eval = NULL,
+    missing = NaN, seed = 0, timeout_request_workers = NULL,
+    checkpoint_path = "", checkpoint_interval = -1,
+    objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
+    num_early_stopping_rounds = 0, objective_type = "classification",
+    eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
+    base_margin_col = NULL,
+    thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
+    prediction_col = "prediction", probability_col = "probability",
+    raw_prediction_col = "rawPrediction",
+    uid = random_string("xgboost_classifier_"), ...) {
   stage <- xgboost_classifier.spark_connection(
     x = spark_connection(x),
     formula = formula,
@@ -248,29 +252,29 @@ xgboost_classifier.ml_pipeline <- function(x, formula = NULL, eta = 0.3, gamma =
 }
 
 #' @export
-xgboost_classifier.tbl_spark <- function(x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
-                                         min_child_weight = 1, max_delta_step = 0,
-                                         grow_policy = "depthwise", max_bins = 16, subsample = 1,
-                                         colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
-                                         alpha = 0, tree_method = "auto", sketch_eps = 0.03,
-                                         scale_pos_weight = 1, sample_type = "uniform",
-                                         normalize_type = "tree", rate_drop = 0, skip_drop = 0,
-                                         lambda_bias = 0, tree_limit = 0, num_round = 1,
-                                         num_workers = 1, nthread = 1, use_external_memory = FALSE,
-                                         silent = 0, custom_obj = NULL, custom_eval = NULL,
-                                         missing = NaN, seed = 0, timeout_request_workers = 30 * 60 * 1000,
-                                         checkpoint_path = "", checkpoint_interval = -1,
-                                         objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
-                                         num_early_stopping_rounds = 0, objective_type = "classification",
-                                         eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
-                                         base_margin_col = NULL,
-                                         thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
-                                         prediction_col = "prediction", probability_col = "probability",
-                                         raw_prediction_col = "rawPrediction",
-                                         uid = random_string("xgboost_classifier_"),
-                                         response = NULL, features = NULL,
-                                         predicted_label_col = "predicted_label", ...) {
-
+xgboost_classifier.tbl_spark <- function(
+    x, formula = NULL, eta = 0.3, gamma = 0, max_depth = 6,
+    min_child_weight = 1, max_delta_step = 0,
+    grow_policy = "depthwise", max_bins = 16, subsample = 1,
+    colsample_bytree = 1, colsample_bylevel = 1, lambda = 1,
+    alpha = 0, tree_method = "auto", sketch_eps = NULL,
+    scale_pos_weight = 1, sample_type = "uniform",
+    normalize_type = "tree", rate_drop = 0, skip_drop = 0,
+    lambda_bias = 0, tree_limit = 0, num_round = 1,
+    num_workers = 1, nthread = 1, use_external_memory = FALSE,
+    silent = 0, custom_obj = NULL, custom_eval = NULL,
+    missing = NaN, seed = 0, timeout_request_workers = NULL,
+    checkpoint_path = "", checkpoint_interval = -1,
+    objective = "multi:softprob", base_score = 0.5, train_test_ratio = 1,
+    num_early_stopping_rounds = 0, objective_type = "classification",
+    eval_metric = NULL, maximize_evaluation_metrics = FALSE, num_class = NULL,
+    base_margin_col = NULL,
+    thresholds = NULL, weight_col = NULL, features_col = "features", label_col = "label",
+    prediction_col = "prediction", probability_col = "probability",
+    raw_prediction_col = "rawPrediction",
+    uid = random_string("xgboost_classifier_"),
+    response = NULL, features = NULL,
+    predicted_label_col = "predicted_label", ...) {
   stage <- xgboost_classifier.spark_connection(
     x = spark_connection(x),
     formula = NULL,
@@ -348,12 +352,16 @@ xgboost_classifier.tbl_spark <- function(x, formula = NULL, eta = 0.3, gamma = 0
 # Validator
 validator_xgboost_classifier <- function(args) {
   args <- validator_xgboost_regressor(args)
-  args[["thresholds"]] <- cast_nullable_double_list(args[["thresholds"]], id = "thresholds") %>%
-    certify(bounded(0, 1), allow_null = TRUE, id = "thresholds")
+  thresholds <- args[["thresholds"]]
+  if (!is.null(thresholds)) {
+    thresholds <- as.list(thresholds)
+  } else if (length(list()) == 0) {
+    thresholds <- NULL
+  }
+  args[["thresholds"]] <- thresholds
   args[["num_class"]] <- cast_nullable_scalar_integer(args[["num_class"]], id = "num_class") %>%
     certify(gte(2), allow_null = TRUE, id = "num_class")
   args[["objective"]] <- cast_choice(args[["objective"]], "multi:softprob")
-
   args
 }
 
@@ -371,7 +379,8 @@ new_xgboost_classification_model <- function(jobj) {
 new_ml_model_xgboost_classification <- function(pipeline_model, formula, dataset, label_col,
                                                 features_col, predicted_label_col) {
   sparklyr::new_ml_model_classification(
-    pipeline_model, formula, dataset = dataset,
+    pipeline_model, formula,
+    dataset = dataset,
     label_col = label_col, features_col = features_col,
     predicted_label_col = predicted_label_col,
     class = "ml_model_xgboost_classification"
@@ -400,3 +409,19 @@ ml_feature_importances.ml_model_xgboost_classification <- function(model, ...) {
 
   result[order(result$importance, decreasing = TRUE), ]
 }
+
+xg_unsupported <- function(args) {
+  if (!is.null(args$sketch_eps)) {
+    stop(
+      "As of XGBoost version 1.6.0, 'Sketch EPS'",
+      " is no longer supported, consider using 'Max Bins'"
+    )
+  }
+  if (!is.null(args$timeout_request_workers)) {
+    stop(
+      "As of XGBoost version 1.7.0, 'Timeout Request Workers'",
+      " is no longer supported"
+    )
+  }
+  invisible()
+}
